﻿using System;
using System.Linq;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;

namespace RobotView
{
    public partial class ConsoleView : UserControl
    {
        private RobotCtrl.Console console;
        private List<LampView> LampViewList;
        private List<SwitchView> SwitchViewList;

        public ConsoleView()
        {
            InitializeComponent();
        }

        public ConsoleView(RobotCtrl.Console console, int numberOfLampsAndSwitches)
            : this()
        {
            this.console = console;

            //initialize lamps and switches
            InitLampsAndSwitches(numberOfLampsAndSwitches);

            //allows console to override 
            this.console.OverrideLamp += console_OverrideLamp;

            //timer timer = new timer();
            //timer.interval = 100;
            //timer.enabled = true;
            //timer.tick += new eventhandler(timer_tick);
            
        }

        void console_OverrideLamp(object sender, RobotCtrl.OverrideLampArgs e)
        {
            this.LampViewList[e.BitIndex].State = e.State;
        }

        /*//The polling approach:
        private void timer_Tick(object sender, EventArgs e)
        {
            console.Lamps.Data = console.Switches.Data;
            for (int i = 0; i < LampViewList.Count; i++)
            {
                //assign lamp control bit to appropriate lamp
                LampViewList[i].State = console.Lamps[i];

                //assign switch control bit to appropriate switch
                SwitchViewList[i].Switch = console.Switches[i];
            }
        }*/

        public void InitLampsAndSwitches(int count)
        {
            this.Size = new Size(0, 0);
            LampViewList = new List<LampView>();
            SwitchViewList = new List<SwitchView>();

            for (int i = 0; i < count; i++)
            {
                Label lbl = new Label();
                lbl.Width = 20;
                lbl.Height = 20;
                lbl.ForeColor = Color.White;
                lbl.BackColor = Color.Black;
                lbl.Text = (i + 1).ToString();
                lbl.Location = new Point(lbl.Width * i, 0);
                this.Controls.Add(lbl);

                LampView lamp = new LampView(console.Lamps, i);
                lamp.Location = new Point(lamp.Width * i, lbl.Height);
                LampViewList.Add(lamp);
                this.Controls.Add(lamp);

                SwitchView swtch = new SwitchView(console.Switches, i);
                swtch.Switched += swtch_Switched;
                swtch.Location = new Point(swtch.Width * i, lbl.Height + lamp.Height);
                SwitchViewList.Add(swtch);
                this.Controls.Add(swtch);

                //set control bounds
                this.Size = new Size(swtch.Width * (i + 1), lbl.Height + lamp.Height + swtch.Height);

            }

            //register hardware event for switches
            this.console.SwitchesChanged += new RobotCtrl.Console.SwitchesChangedEventHandler(console_SwitchesChanged);
        }

        //Software interrupt (click on button)
        //This is the "more smooth way" to assign switch values to lamps.
        private void swtch_Switched(object sender, EventArgs e)
        {
            /*
            SwitchView switchView = (SwitchView)sender;

            //assign switch value to console
            console.Lamps[switchView.BitIndex] = console.Switches[switchView.BitIndex];

            //assign lamp control bit to appropriate lamp
            LampViewList[switchView.BitIndex].State = console.Lamps[switchView.BitIndex];

            //assign switch control bit to appropriate switch
            SwitchViewList[switchView.BitIndex].Switch = console.Switches[switchView.BitIndex];
            */

            performSwitch();

        }

        //Hardware interrupts
        //Unfortunately there are no real hardware interrupts
        //on our embedded board. Thus we have to loop through each switch/lamp bit
        void console_SwitchesChanged(System.Collections.Specialized.BitVector32 oldValue, System.Collections.Specialized.BitVector32 newValue)
        {
            performSwitch();
        }

        private void performSwitch()
        {
            //console.Lamps.Data = console.Switches.Data;
            for (int i = 0; i < LampViewList.Count; i++)
            {
                console.Lamps[i] = console.Switches[i];

                //assign lamp control bit to appropriate lamp
                LampViewList[i].State = console.Lamps[i];

                //assign switch control bit to appropriate switch
                SwitchViewList[i].Switch = console.Switches[i];
            }
        }
    }
}
